@@ -10,15 +10,20 @@ import operator
 import sys
 import time
 
+from numbers import Integral, Real
 import numpy as np
 from scipy import linalg
-from joblib import Parallel
 
 from . import empirical_covariance, EmpiricalCovariance, log_likelihood
 
 from ..exceptions import ConvergenceWarning
-from ..utils.validation import _is_arraylike_not_scalar, check_random_state
-from ..utils.fixes import delayed
+from ..utils.validation import (
+    _is_arraylike_not_scalar,
+    check_random_state,
+    check_scalar,
+)
+from ..utils.parallel import delayed, Parallel
+from ..utils._param_validation import Interval, StrOptions
 
 # mypy error: Module 'sklearn.linear_model' has no attribute '_cd_fast'
 from ..linear_model import _cd_fast as cd_fast  # type: ignore
@@ -72,29 +77,6 @@ def alpha_max(emp_cov):
     return np.max(np.abs(A))
 
 
-class _DictWithDeprecatedKeys(dict):
-    """Dictionary with deprecated keys.
-
-    Currently only be used in GraphicalLassoCV to deprecate keys"""
-
-    def __init__(self, **kwargs):
-        super().__init__(**kwargs)
-        self._deprecated_key_to_new_key = {}
-
-    def __getitem__(self, key):
-        if key in self._deprecated_key_to_new_key:
-            warnings.warn(
-                f"Key: '{key}', is deprecated in 1.0 and will be "
-                f"removed in 1.2. Use '{self._deprecated_key_to_new_key[key]}' instead",
-                FutureWarning,
-            )
-        return super().__getitem__(key)
-
-    def _set_deprecated(self, value, *, new_key, deprecated_key):
-        self._deprecated_key_to_new_key[deprecated_key] = new_key
-        self[new_key] = self[deprecated_key] = value
-
-
 # The g-lasso algorithm
 def graphical_lasso(
     emp_cov,
@@ -153,7 +135,7 @@ def graphical_lasso(
         If verbose is True, the objective function and dual gap are
         printed at each iteration.
 
-    return_costs : bool, default=Flase
+    return_costs : bool, default=False
         If return_costs is True, the objective function and dual gap
         at each iteration are returned.
 
@@ -271,8 +253,7 @@ def graphical_lasso(
                             check_random_state(None),
                             False,
                         )
-                    else:
-                        # Use LARS
+                    else:  # mode == "lars"
                         _, _, coefs = lars_path_gram(
                             Xy=row,
                             Gram=sub_covariance,
@@ -334,7 +315,35 @@ def graphical_lasso(
             return covariance_, precision_
 
 
-class GraphicalLasso(EmpiricalCovariance):
+class BaseGraphicalLasso(EmpiricalCovariance):
+    _parameter_constraints: dict = {
+        **EmpiricalCovariance._parameter_constraints,
+        "tol": [Interval(Real, 0, None, closed="right")],
+        "enet_tol": [Interval(Real, 0, None, closed="right")],
+        "max_iter": [Interval(Integral, 0, None, closed="left")],
+        "mode": [StrOptions({"cd", "lars"})],
+        "verbose": ["verbose"],
+    }
+    _parameter_constraints.pop("store_precision")
+
+    def __init__(
+        self,
+        tol=1e-4,
+        enet_tol=1e-4,
+        max_iter=100,
+        mode="cd",
+        verbose=False,
+        assume_centered=False,
+    ):
+        super().__init__(assume_centered=assume_centered)
+        self.tol = tol
+        self.enet_tol = enet_tol
+        self.max_iter = max_iter
+        self.mode = mode
+        self.verbose = verbose
+
+
+class GraphicalLasso(BaseGraphicalLasso):
     """Sparse inverse covariance estimation with an l1-penalized estimator.
 
     Read more in the :ref:`User Guide <sparse_inverse_covariance>`.
@@ -430,6 +439,11 @@ class GraphicalLasso(EmpiricalCovariance):
     array([0.073, 0.04 , 0.038, 0.143])
     """
 
+    _parameter_constraints: dict = {
+        **BaseGraphicalLasso._parameter_constraints,
+        "alpha": [Interval(Real, 0, None, closed="right")],
+    }
+
     def __init__(
         self,
         alpha=0.01,
@@ -441,13 +455,15 @@ class GraphicalLasso(EmpiricalCovariance):
         verbose=False,
         assume_centered=False,
     ):
-        super().__init__(assume_centered=assume_centered)
+        super().__init__(
+            tol=tol,
+            enet_tol=enet_tol,
+            max_iter=max_iter,
+            mode=mode,
+            verbose=verbose,
+            assume_centered=assume_centered,
+        )
         self.alpha = alpha
-        self.mode = mode
-        self.tol = tol
-        self.enet_tol = enet_tol
-        self.max_iter = max_iter
-        self.verbose = verbose
 
     def fit(self, X, y=None):
         """Fit the GraphicalLasso model to X.
@@ -465,6 +481,7 @@ class GraphicalLasso(EmpiricalCovariance):
         self : object
             Returns the instance itself.
         """
+        self._validate_params()
         # Covariance does not make sense for a single feature
         X = self._validate_data(X, ensure_min_features=2, ensure_min_samples=2)
 
@@ -606,7 +623,7 @@ def graphical_lasso_path(
     return covariances_, precisions_
 
 
-class GraphicalLassoCV(GraphicalLasso):
+class GraphicalLassoCV(BaseGraphicalLasso):
     """Sparse inverse covariance w/ cross-validated choice of the l1 penalty.
 
     See glossary entry for :term:`cross-validation estimator`.
@@ -622,7 +639,8 @@ class GraphicalLassoCV(GraphicalLasso):
         If an integer is given, it fixes the number of points on the
         grids of alpha to be used. If a list is given, it gives the
         grid to be used. See the notes in the class docstring for
-        more details. Range is (0, inf] when floats given.
+        more details. Range is [1, inf) for an integer.
+        Range is (0, inf] for an array-like of floats.
 
     n_refinements : int, default=4
         The number of times the grid is refined. Not used if explicit
@@ -718,29 +736,6 @@ class GraphicalLassoCV(GraphicalLasso):
 
             .. versionadded:: 1.0
 
-        split(k)_score : ndarray of shape (n_alphas,)
-            Log-likelihood score on left-out data across (k)th fold.
-
-            .. deprecated:: 1.0
-                `split(k)_score` is deprecated in 1.0 and will be removed in 1.2.
-                Use `split(k)_test_score` instead.
-
-        mean_score : ndarray of shape (n_alphas,)
-            Mean of scores over the folds.
-
-            .. deprecated:: 1.0
-                `mean_score` is deprecated in 1.0 and will be removed in 1.2.
-                Use `mean_test_score` instead.
-
-        std_score : ndarray of shape (n_alphas,)
-            Standard deviation of scores over the folds.
-
-            .. deprecated:: 1.0
-                `std_score` is deprecated in 1.0 and will be removed in 1.2.
-                Use `std_test_score` instead.
-
-        .. versionadded:: 0.24
-
     n_iter_ : int
         Number of iterations run for the optimal alpha.
 
@@ -798,6 +793,14 @@ class GraphicalLassoCV(GraphicalLasso):
     array([0.073, 0.04 , 0.038, 0.143])
     """
 
+    _parameter_constraints: dict = {
+        **BaseGraphicalLasso._parameter_constraints,
+        "alphas": [Interval(Integral, 1, None, closed="left"), "array-like"],
+        "n_refinements": [Interval(Integral, 1, None, closed="left")],
+        "cv": ["cv_object"],
+        "n_jobs": [Integral, None],
+    }
+
     def __init__(
         self,
         *,
@@ -813,11 +816,11 @@ class GraphicalLassoCV(GraphicalLasso):
         assume_centered=False,
     ):
         super().__init__(
-            mode=mode,
             tol=tol,
-            verbose=verbose,
             enet_tol=enet_tol,
             max_iter=max_iter,
+            mode=mode,
+            verbose=verbose,
             assume_centered=assume_centered,
         )
         self.alphas = alphas
@@ -841,6 +844,7 @@ class GraphicalLassoCV(GraphicalLasso):
         self : object
             Returns the instance itself.
         """
+        self._validate_params()
         # Covariance does not make sense for a single feature
         X = self._validate_data(X, ensure_min_features=2)
         if self.assume_centered:
@@ -857,6 +861,15 @@ class GraphicalLassoCV(GraphicalLasso):
         inner_verbose = max(0, self.verbose - 1)
 
         if _is_arraylike_not_scalar(n_alphas):
+            for alpha in self.alphas:
+                check_scalar(
+                    alpha,
+                    "alpha",
+                    Real,
+                    min_val=0,
+                    max_val=np.inf,
+                    include_boundaries="right",
+                )
             alphas = self.alphas
             n_refinements = 1
         else:
@@ -958,26 +971,13 @@ class GraphicalLassoCV(GraphicalLasso):
         )
         grid_scores = np.array(grid_scores)
 
-        # TODO(1.2): Use normal dict for cv_results_ instead of _DictWithDeprecatedKeys
-        self.cv_results_ = _DictWithDeprecatedKeys(alphas=np.array(alphas))
+        self.cv_results_ = {"alphas": np.array(alphas)}
 
         for i in range(grid_scores.shape[1]):
-            self.cv_results_._set_deprecated(
-                grid_scores[:, i],
-                new_key=f"split{i}_test_score",
-                deprecated_key=f"split{i}_score",
-            )
+            self.cv_results_[f"split{i}_test_score"] = grid_scores[:, i]
 
-        self.cv_results_._set_deprecated(
-            np.mean(grid_scores, axis=1),
-            new_key="mean_test_score",
-            deprecated_key="mean_score",
-        )
-        self.cv_results_._set_deprecated(
-            np.std(grid_scores, axis=1),
-            new_key="std_test_score",
-            deprecated_key="std_score",
-        )
+        self.cv_results_["mean_test_score"] = np.mean(grid_scores, axis=1)
+        self.cv_results_["std_test_score"] = np.std(grid_scores, axis=1)
 
         best_alpha = alphas[best_index]
         self.alpha_ = best_alpha
